@@ -1,25 +1,18 @@
 
 from __future__ import with_statement
-import os, sys
+import os
+import sys
 import shlex
 import hashlib
-import mimetypes
-import posixpath
 import py
-import devpi
-import argparse
-import pkg_resources
-import archive
+from devpi_common.archive import Archive
 import json
 import tox
 
-from devpi.util import url as urlutil
-from devpi.util import version as verlib
-from devpi.util import pypirc
+from devpi_common.url import URL
+from devpi_common.metadata import splitbasename
 from devpi.remoteindex import RemoteIndex
 
-import requests
-
 def setenv_devpi(hub, env, posturl, packageurl, packagemd5):
     if not packagemd5:
         packagemd5 = ""
@@ -45,18 +38,19 @@ class DevIndex:
 
     def download_and_unpack(self, link):
         try:
-            content = self.remoteindex.getcontent(link.href)
+            content = self.remoteindex.getcontent(link.url, bytes=True)
         except self.remoteindex.ReceiveError:
-            self.hub.fatal("could not receive", link.href)
+            self.hub.fatal("could not receive", link.url)
 
-        self.hub.info("received", link.href)
+        self.hub.info("received", link.url)
         if hasattr(link, "md5"):
             md5 = hashlib.md5()
             md5.update(content)
             digest = md5.hexdigest()
             assert digest == link.md5, (digest, link.md5)
             #self.hub.info("verified md5 ok", link.md5)
-        path_archive = self.dir_download.join(link.basename)
+        basename = URL(link.url).basename
+        path_archive = self.dir_download.join(basename)
         with path_archive.open("wb") as f:
             f.write(content)
         pkg = UnpackedPackage(self.hub, self.rootdir, path_archive, link)
@@ -68,31 +62,27 @@ class DevIndex:
         return self.remoteindex.getbestlink(pkgname)
 
     def runtox(self, link):
-        path_archive = link.pkg.path_archive
-
         # publishing some infos to the commands started by tox
         #setenv_devpi(self.hub, env, posturl=self.current.resultlog,
-        #                  packageurl=link.href,
+        #                  packageurl=link.url,
         #                  packagemd5=link.md5)
         jsonreport = link.pkg.rootdir.join("toxreport.json")
+        path_archive = link.pkg.path_archive
         toxargs = ["--installpkg", str(path_archive),
                    "-i ALL=%s" % str(self.current.simpleindex),
                    "--result-json", str(jsonreport),
         ]
-        toxargs.extend(self.get_tox_args())
+        unpack_path = link.pkg.path_unpacked
 
-        self.hub.info("%s$ tox %s" %(link.pkg.path_unpacked,
-                      " ".join(toxargs)))
-        old = link.pkg.path_unpacked.chdir()
-        try:
+        toxargs.extend(self.get_tox_args(unpack_path=unpack_path))
+        with link.pkg.path_unpacked.as_cwd():
+            self.hub.info("%s$ tox %s" %(os.getcwd(), " ".join(toxargs)))
             try:
                 ret = tox.cmdline(toxargs)
             except SystemExit as e:
                 ret = e.args[0]
-        finally:
-            old.chdir()
         if ret != 2:
-            jsondata = json.load(jsonreport.open("rb"))
+            jsondata = json.load(jsonreport.open("r"))
             post_tox_json_report(self.hub,
                                  self.hub.current.resultlog, jsondata)
         if ret != 0:
@@ -100,11 +90,21 @@ class DevIndex:
             return 1
         return 0
 
-    def get_tox_args(self):
-        toxargs = []
+    def get_tox_args(self, unpack_path):
+        hub = self.hub
         args = self.hub.args
+        toxargs = []
         if args.venv is not None:
             toxargs.append("-e" + args.venv)
+        if args.toxini:
+            ini = hub.get_existing_file(args.toxini)
+        elif unpack_path.join("tox.ini").exists():
+            ini = hub.get_existing_file(unpack_path.join("tox.ini"))
+        elif args.fallback_ini:
+            ini = hub.get_existing_file(args.fallback_ini)
+        else:
+            hub.fatal("no tox.ini file found in %s" % unpack_path)
+        toxargs.extend(["-c", str(ini)])
         if args.toxargs:
             toxargs.extend(shlex.split(args.toxargs))
         return toxargs
@@ -126,8 +126,10 @@ class UnpackedPackage:
 
     def unpack(self):
         self.hub.info("unpacking", self.path_archive, "to", str(self.rootdir))
-        archive.extract(str(self.path_archive), to_path=str(self.rootdir))
-        pkgname, version = verlib.guess_pkgname_and_version(self.link.basename)
+        with Archive(self.path_archive) as archive:
+            archive.extract(self.rootdir)
+        basename = URL(self.link.url).basename
+        pkgname, version = splitbasename(basename)[:2]
         subdir = "%s-%s" %(pkgname, version)
         inpkgdir = self.rootdir.join(subdir)
         assert inpkgdir.check(), inpkgdir
